// if you haven't already done so, you will need to download oscP5
//http://www.sojamo.de/libraries/oscP5/
import oscP5.*; 
import netP5.*;


int counter=0; 
float stretch; 

//OSC stuff
OscP5 oscP5;
NetAddress myRemoteLocation;


void setup() {
  size(400, 400, JAVA2D);

  frameRate(10); // how many times screen refreshed per second

// then declare instances of OSC classes - we need to specify
// a) address and port to listen on
// b) address and port to send on

  /* start oscP5, listening for incoming messages at port 12000 */
  oscP5 = new OscP5(this, 12000);  // so this must match the port specified in new NetAddr instance in SC

  /* myRemoteLocation is a NetAddress. a NetAddress takes 2 parameters,
   * an ip address and a port number. myRemoteLocation is used as parameter in
   * oscP5.send() when sending osc packets to another computer, device, 
   * application. usage see below. 
   */

  //57120 is SC lang default port
  myRemoteLocation = new NetAddress("127.0.0.1", 57121);
  
  stretch=random(0.5, 2.5);    // We'll set this var in SC and send it in ...  
}



void draw() {

  counter = (counter+1)%6;

  if (counter == 0) {

    //in the following different ways of creating osc messages are shown by example 
    OscMessage myMessage = new OscMessage("/test");

    myMessage.add(stretch+60); // add a float to the osc message 

    // send the message 
    oscP5.send(myMessage, myRemoteLocation);
  }

  background(255);

  strokeWeight(random(1, 5));

  stroke(40*stretch, 30, 20); // this sets the colour
  
  line(20, 20, stretch*40, stretch*10); // and draw X,Y coords of the line according to incoming value
  
}





/* incoming osc message are forwarded to the oscEvent method. */
// This is the equivalent of OSCFunc in SC
// we can also 'plug' messsages which allows us to forward OSc msgs with a specific addrPattern straight to a method of the same name. 
// more on this next week. This is the same as OScFunc. 

void oscEvent(OscMessage theOscMessage) {
  /* print the address pattern and the typetag of the received OscMessage */
  print("### received an osc message.");
  print(" addrpattern: "+theOscMessage.addrPattern());
  println(" typetag: "+theOscMessage.typetag());
  println(" arguments: [0] "+theOscMessage.get(0).intValue()+" [1] "+theOscMessage.get(1).stringValue()+" [2] "+theOscMessage.get(2).floatValue());
// assign the third element in the message array (the actual value) to the stretch variable
  stretch= theOscMessage.get(2).floatValue();
}


