// Based on a sketch by Memo Akten, May 2011. http://www.memo.tv
// which in turn was based on "Pendulum Waves" http://j.mp/lxZfTY
// including all numbers (0.85Hz for slowest pendulum, 1.083Hz for fastest)
// if the above shortened link dies, try: http://sciencedemonstrations.fas.harvard.edu/icb/icb.do?keyword=k16940&pageid=icb.page80863&pageContentId=icb.pagecontent341734&state=maximize&view=view.do&viewParam_name=indepth.html#a_icb_pagecontent341734



import oscP5.*; // if you haven't already done so, you will need to download oscP5
import netP5.*;
//OSC stuff
OscP5 oscP5;
NetAddress myRemoteLocation;



final int numPendulums = 8;
Pendulum[] pendulums;

float startTime;


int recPort;  //  port P5 listens on 
int sendPort; //  port P5 sends over 


// Let's use a chromatic scale to start
int musicscale[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11};
int scaleNoteCount = 12;

// we could use pentatonic ... or something more interesting ... 
//int musicscale[] = {0, 3, 5, 7, 10};
//int scaleNoteCount = 5;



//-------------------------------------
// All P5 sketches have a setup() function - it is where we set things up !!
//-------------------------------------
void setup() {
  size(350, 720);
  background(0);
  frameRate(60);
  noStroke();

  recPort = 12000;    // this is the port P5 listens on - must match the port specified in the NetAddr instance in SC
  sendPort = 57121;  // this is the port P5 sends over - currently SC default
  
  oscP5 = new OscP5(this, recPort);  
  myRemoteLocation = new NetAddress("127.0.0.1", sendPort);
  
  
  // create array of pendulums
  pendulums = new Pendulum[numPendulums];

  // loop through and init each pendulum
  for (int c=0; c<numPendulums; c++) {
    // first pendulum oscillates 51 times per minute
    // each successive has one more oscillation per minute
    float freq = (51.0 + c)/60.0;

    // pick the next midi note from the musicscale array, wrapping around (and increasing octave) as neccesary
    // NB - we could explore more interesting scales and tuning systems and send as freq (floats)
    int midiNote = musicscale[c%scaleNoteCount] + (c/scaleNoteCount)*12 + 60;

    // init pendulum instance
    pendulums[c] = new Pendulum(c, freq, midiNote);
  }

  // save time we have finished the setup, to ensure pendulums start at t==0
  startTime = millis() * 0.001;
}


//-------------------------------------
// draw() is also a default P5 function - it is called automatically every X FPS 
// - framerate default is 60 fps and can be set in setup()
//-------------------------------------
void draw() {
  background(0);

  // size to draw pendulum
  float pendulumSize = height * 1.0 / numPendulums;

  // seconds since we started the app (minus setup time)
  float secs = millis() * 0.001 - startTime;

  // loop through, update and draw pendulums
  for (int c=0; c<numPendulums; c++) {
    pendulums[c].update(secs);
    pendulums[c].draw(map(c, 0, numPendulums-1, pendulumSize/2, height - pendulumSize/2), pendulumSize);
  }
}


//-------------------------------------
// This is called by instances of the pendulum class when they hit the side
// Currently we are only sending 1 value but could be expanded to include more
//-------------------------------------
void sendTrigger(int channel, int note, int pan) {
//  int velocity = (int)random(50, 100);
  
  OscMessage myMessage = new OscMessage("/play");
  myMessage.add(note); // add an int to the osc message 
  myMessage.add(pan);

  // send the message 
  oscP5.send(myMessage, myRemoteLocation);
  println(note);
}


//-------------------------------------
void keyPressed() {
  startTime = millis() * 0.001;
}

