//-------------------------------------
// Pendulum class makes it easy to create multiple instances with different parameters
//-------------------------------------

class Pendulum {
  // physical params
  int index;
  float freq;      // oscilations per second
 
  // physical vars
  float pos;       // position of pendulum (-1...1)
  float vel;       // current velocity
 
  // musical params
  int pitch;      // midi note number to trigger
 
  // display params
  float hit;      // 0...1, how recently we 'hit' the edge
 
  // constructor
  Pendulum(int _index, float _freq, int _pitch) {
    index = _index;
    freq = _freq;
    pitch = _pitch;
    pos = 1;
    vel = 0;
    hit = 0;
  }
 
  // update position and trigger sound if neccesary
  void update(float t) {
    float oldPos = pos;     // save old position
    float oldVel = vel;     // save old direction
 
    pos = cos(freq * t * 2 * PI);  // calculate new position
    vel = pos - oldPos;            // calculate new direction
 
    if (vel * oldVel <= 0.0) {     // if pendulum has changed direction...
      sendTrigger(index, pitch, pos < 0 ? 0 : 127);
      hit = 1;
    }
    else {
      hit *= 0.85;  // hit used to fade out dot
    }
  }
 
  // draw
  void draw(float y, float size) {
    // set color
    float r = lerp(30, 0, hit);     // performs linear interpolation to control RGB vals to fade colour
    float g = lerp(30, 220, hit);
    float b = lerp(30, 180, hit);
    fill(r, g, b);
 
    // draw circle - scaled according to nPendulums
    ellipse(map(pos, -1, 1, size/2, width-size/2), y, size, size);
  }
};
 
